@@ -79,6 +79,8 @@ struct _GProxyResolverGnome {
   gchar *ftp_proxy, *socks_authority;
 
   GDBusProxy *pacrunner;
+
+  GMutex *lock;
 };
 
 static void g_proxy_resolver_gnome_iface_init (GProxyResolverInterface *iface);
@@ -114,7 +116,9 @@ gsettings_changed (GSettings   *settings,
 {
   GProxyResolverGnome *resolver = user_data;
 
+  g_mutex_lock (resolver->lock);
   resolver->need_update = TRUE;
+  g_mutex_unlock (resolver->lock);
 }
 
 static void
@@ -155,12 +159,16 @@ g_proxy_resolver_gnome_finalize (GObject *object)
   if (resolver->pacrunner)
     g_object_unref (resolver->pacrunner);
 
+  g_mutex_free (resolver->lock);
+
   G_OBJECT_CLASS (g_proxy_resolver_gnome_parent_class)->finalize (object);
 }
 
 static void
 g_proxy_resolver_gnome_init (GProxyResolverGnome *resolver)
 {
+  resolver->lock = g_mutex_new ();
+
   resolver->proxy_settings = g_settings_new (GNOME_PROXY_SETTINGS_SCHEMA);
   g_signal_connect (resolver->proxy_settings, "changed",
 		    G_CALLBACK (gsettings_changed), resolver);
@@ -180,6 +188,7 @@ g_proxy_resolver_gnome_init (GProxyResolverGnome *resolver)
   resolver->need_update = TRUE;
 }
 
+/* called with lock held */
 static void
 update_settings (GProxyResolverGnome *resolver)
 {
@@ -303,7 +312,8 @@ update_settings (GProxyResolverGnome *resolver)
       GError *error = NULL;
       resolver->pacrunner =
 	g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
-				       G_DBUS_PROXY_FLAGS_NONE,
+				       G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES |
+				       G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
 				       NULL,
 				       "org.gtk.GLib.PACRunner",
 				       "/org/gtk/GLib/PACRunner",
@@ -448,8 +458,10 @@ g_proxy_resolver_gnome_lookup (GProxyResolver  *proxy_resolver,
   const gchar *proxy = "direct://";
   gchar **proxies = NULL;
 
+  g_mutex_lock (resolver->lock);
   if (resolver->need_update)
     update_settings (resolver);
+  g_mutex_unlock (resolver->lock);
 
   if (resolver->mode == G_DESKTOP_PROXY_MODE_NONE)
     goto done;
